/* 
   COPYRIGHT 2004 KRUPCZAK.ORG, LLC.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA
 
   For more information, visit:
   http://www.krupczak.org/
*/

/* file - housekeeping.c */
/* rdk@krupczak.org */

/* Daemon's housekeeping functions */

#ifdef WIN32
#include <windows.h>
#endif

#include <pthread.h>
#include <stdio.h>
#include <sys/types.h>

#ifndef WIN32
#include <sys/time.h>
#include <unistd.h>
#endif

#include "xmp.h"
#include "globals.h"
#include "plugins.h"

/* ************************************************************* */
/* do periodic processing, cleanup, etc. */

void *houseKeeping(void *arg)
{
     int i;

     while (globalShutdown == 0) {

       if (xmpd_update_configfile) {
	  writeXmpdConfigFile();
	  xmpd_update_configfile = 0;
       }

       for (i=0; i<2 && globalShutdown == 0; i++) {
#ifndef WIN32
          sleep(15);
#else
          Sleep(15*1000);
#endif
       }
       if (globalShutdown == 1) break;

       cleanupAllPlugins();
       findLoadAllPlugins(xmpd_home_dir);

       /* check status of config file; re-read if necessary XXX */

       processEvents();

       /* check for upgrade */
       upgradeCartographer();

       DBG("houseKeeping: working . . .\n");

     }

     xmpd_log("housekeeping finishing, unloading plugins");
     fprintf(stderr,"housekeeping finishing, unloading plugins\n");

     /* unload everything */
     unloadAllPlugins();

#ifndef WIN32
     sleep(5);
#else
     Sleep(5*1000);
#endif

     xmpd_log("housekeeping done");

     return NULL;
}

/* ************************************************************* */
