/* 
   COPYRIGHT 2004 KRUPCZAK.ORG, LLC.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA
 
   For more information, visit:
   http://www.krupczak.org/
*/

/* file - linkedlist.h */
/* rdk@krupczak.org */

/* structures for generic doubly linked list package */

/* ************************************************************* */

typedef struct linkedElement {

     void *data;

     struct linkedElement *next;
     struct linkedElement *prev;

} linkedElement_t;

typedef struct linkedList {
   
     char description[64];
     
     linkedElement_t *head;
     linkedElement_t *tail;

     pthread_mutex_t lock;

     /* size data, in bytes, that user will glue into elements */
     int dataSize;

     /* compare function used for insertInOrder; returns -1,0,1 */
     int (*elementCompare)();
     int (*elementPrint)();

     /* for freeing user data; if not defined, we do *not* free
        any user-defined data */
     int (*elementFree)(); 

} linkedList_t;

linkedList_t *newLinkedList(int, void *, void *, void *, char *);
int destroyLinkedList(linkedList_t *);
int insertInOrder(linkedList_t *, void *);
int insertHead(linkedList_t *, void *);
int insertTail(linkedList_t *, void *);
int destroyElement(linkedList_t *, linkedElement_t *);
void *popHead(linkedList_t *);
void *popTail(linkedList_t *);
int numberElements(linkedList_t *);
int traverseList(linkedList_t *);
linkedElement_t *findElement(linkedList_t *, void *);
u_char *toArrayCopy(linkedList_t *, int *);
int iterateList(linkedList_t *, int (*)());
int drainLinkedList(linkedList_t *list);
int removeElement(linkedList_t *, linkedElement_t *);
int disconnectElement(linkedList_t *, linkedElement_t *);

/* ************************************************************* */
